// Endogenous Variables
var 
c                 // total consumption
l                 // aggregate employment
v                 // aggregate vacancies
w                 // wage
k                 // aggregate capital
ik                // investment in physical capital
zc                // job-productivity cutoff
q                 // probability of filling a vacancy
m                 // matches
s                 // job-finding probability
u                 // unemployment rate
zav               // average job productivity
rho               // endogenous separation rate
mpl               // marginal product of labor
mpk               // marginal product of capital
phi               // value of a match to the firm
surplusF          // firm surplus
surplusW          // worker surplus
oo                // value of unemployment
wzc               // wage marginal worker
y                 // aggregate output
rk                // real rate for capital
mc                // marginal cost
multR             // investment lagrange multiplier
adjI              // investment adjustment cost  
e                 // firm value
d                 // per-period firm profit
N                 // number of producers
Ne                // number of new entrants
rpD               // relative price 
elasticity        // elasticity of substitution
markup            // markup
yfirm             // firm output
yn                // firm production funtion
vfirm             // firm vacancies
lfirm             // firm employment
kfirm             // firm capital
//Util              // household's utility
//Welf              // household's welfare
uc                // marginal utility of consumption
zavInt            // integral to compute average job productivity
mplzc             // marginal product of marginal worker
seprate           // separation rate
cm                // market consumption
ie                // investment in new products 
z                 // aggregate productivity
gdpIncome         // GDP Income
gdpIncomedata     // GDP Income data-consistent
cmdata            // market consumption data-consistent
ikdata            // investment in physical capital data-consistent
iedata            // investment in new products data-consistent 
wdata             // wage data-consistent
cdata             // total consumption data-consistent
ub                // unemployment benefit
fc                // firing cost
fr                // sunk entry cost regulation
tau
profitsub
d_nosub
;


// Innovations
varexo epsZ; 

parameters   gammaP betaP rhozP stdzP lambdaP etaP epsP meanziP stdziP alphaP piP adjIP deltafP  NbarP sigmaMuP chiP vcP fetechP deltakP shapeP exponenpfP fetechtarget hpP,
           c_SS l_SS v_SS w_SS  k_SS ik_SS zc_SS q_SS m_SS s_SS u_SS zav_SS rho_SS mpl_SS mpk_SS phi_SS surplusF_SS surplusW_SS oo_SS wzc_SS y_SS rk_SS mc_SS multR_SS adjI_SS
           e_SS d_SS N_SS Ne_SS rpD_SS elasticity_SS markup_SS yfirm_SS yn_SS vfirm_SS lfirm_SS kfirm_SS uc_SS  zavInt_SS
           surplusFzc_SS surplusWzc_SS phizc_SS mplzc_SS cm_SS ub_SS fc_SS fr_SS z_SS gdpIncome_SS tau_SS profitsub_SS d_nosub_SS;
          // c_SSinit l_SSinit  k_SSinit q_SSinit  N_SSinit Ne_SSinit z_SSinit  ik_SSinit c_SSinit lfirm_SSinit vfirm_SSinit  fr_SSinit ub_SSinit fc_SSinit;


set_parameters_from_file('steady',M_.param_names);
//set_parameters_from_file('steadyInitial',M_.param_names);
set_parameters_from_file('calibration',M_.param_names);

// The initval values in OE_optimal_nos_cycle_irf are from running the non-exp(log) version of cycle.mod

model;


// %---Utility Function
//Util              = log(exp(c));

//Welf              = Util + betaP*Welf(1);


//% Labor Market Block
- exp(lfirm)      + (1-lambdaP)*(1-exp(rho))*exp(lfirm(-1)) + (1-lambdaP)*(1-exp(rho))*exp(q(-1))*exp(vfirm(-1));

- vcP/exp(q)      + (1-lambdaP)*(1-deltafP)*(betaP*exp(uc(1))/exp(uc))*((1 - exp(rho(1)))*exp(phi(1)) - exp(rho(1))*fc);  

- exp(mplzc)      + exp(wzc) - vcP/exp(q)  - fc;

- exp(surplusF)   + ( (1 - etaP)/etaP )*exp(surplusW);

- exp(wzc)        + exp(w) - etaP*(exp(mpl) - exp(mplzc)); 

- exp(yn)         + exp(z)*(exp(zav)^shapeP)*(exp(k(-1))^alphaP)*(exp(l)^(1-alphaP));

//%% Entry Block
//- exp(d)          + exp(rpD)*exp(yfirm) - exp(w)*exp(lfirm) - vcP*exp(vfirm) - exp(mpk)*exp(kfirm) - exp(rho)*exp(lfirm)*fc/(1-exp(rho));  
 
- exp(profitsub)  + (1/(2*sigmaMuP*exp(N)))*exp(rpD)*exp(yfirm);

- exp(d)          + exp(rpD)*exp(yfirm) - exp(w)*exp(lfirm) - vcP*exp(vfirm) - exp(mpk)*exp(kfirm) - exp(rho)*exp(lfirm)*fc/(1-exp(rho)) + exp(profitsub);

- exp(e)          + (fetechP + fr) + vcP*(exp(lfirm)/exp(q) + exp(vfirm)) ;

- exp(yfirm)      + exp(yn)/exp(N);

//- exp(rpD)        + exp(markup)*exp(mc);

- exp(tau)        + exp(markup) - 1;

- exp(mc)         + exp(rpD)/exp(markup)*(1 + exp(tau));

- exp(e)          + (1 - deltafP)*betaP*(exp(uc(1))/exp(uc))*(exp(e(+1)) + exp(d(+1)));

- exp(N)          + (1 - deltafP )*(exp(N(-1)) + exp(Ne(-1)));

- 1               + exp(rpD)*((exp(.5*(NbarP - exp(N))/(sigmaMuP*NbarP*exp(N))))) ;

- exp(markup)     + exp(elasticity)/(exp(elasticity) - 1);

- exp(elasticity) + ((1 + sigmaMuP*exp(N)));

- exp(yfirm)      + (1/exp(rpD))*exp(y)/exp(N);

- exp(y)          + exp(cm) + exp(ik) + vcP*exp(v) + exp(Ne)*(fetechP + fr) + exp(rho)*exp(lfirm)*exp(N)*fc/(1-exp(rho));  


//%% Capital Accumulation 

- exp(k)          + (1 - deltakP)*exp(k(-1))   +  exp(ik)*(1 - (adjIP/2)*(adjI^2));

- exp(multR)      + (betaP*exp(uc(1))/exp(uc))*(exp(rk(1)) + (1 - deltakP)*exp(multR(1)));

- 1               + exp(multR)*((1 - (adjIP/2)*(adjI^2))  -  adjIP*adjI*(exp(ik)/exp(ik(-1))))  + adjIP*(betaP*exp(uc(1))/exp(uc))*exp(multR(1))*(adjI(1)*((exp(ik(1))/exp(ik))^2));

- adjI            + exp(ik)/exp(ik(-1)) - 1;


//% Definitions (required to solve the model)

- exp(phi)        + exp(mpl) - exp(w) + (1-lambdaP)*(1-deltafP)*(betaP*exp(uc(1))/exp(uc))*( (1-exp(rho(1)))*exp(phi(1)) - exp(rho(1))*fc(1) );  

- exp(surplusW)   + exp(w) - exp(oo) + (1-lambdaP)*(1-deltafP)*(betaP*exp(uc(1))/exp(uc))*(1-exp(rho(1)))*exp(surplusW(1)); 

- exp(oo)         + hpP + ub  + (1-lambdaP)*(1-deltafP)*betaP*(exp(uc(1))/exp(uc))*(exp(s))*(1-exp(rho(1)))*exp(surplusW(1));

- exp(v)          + (exp(N) + exp(Ne))*exp(vfirm) + exp(Ne)*exp(lfirm)/exp(q) ; 

- exp(lfirm)      + exp(l)/exp(N); 

- exp(kfirm)      + exp(k(-1))/exp(N); 

- exp(m)          + (chiP*(exp(v)^(1-epsP))*(exp(u)^(epsP)));

- exp(u)          + 1 - exp(l);

- exp(s)          + exp(m)/exp(u);

- exp(q)          + exp(m)/exp(v);

- exp(zavInt)     + (-1/2*piP^(1/2)*exp(1/2*(2*meanziP+stdziP^2*shapeP)*shapeP/(1-2*alphaP+alphaP^2))*(erf(1/2*2^(1/2)/stdziP*(-log(exp(zc))+log(exp(zc))*alphaP+stdziP^2*shapeP+meanziP-meanziP*alphaP)/(-1+alphaP))-erf(1/2*2^(1/2)*(-10000000000+10000000000*alphaP+stdziP^2*shapeP+meanziP-meanziP*alphaP)/stdziP/(-1+alphaP)))*exp(-shapeP*meanziP*alphaP/(-1+alphaP)^2)/piP^(1/2))/(1-exp(rho));

- exp(zav)        + exp(zavInt)^((1 - alphaP)/shapeP);

- exp(rho)        + ((0.5 + 0.5 * erf((log(exp(zc)))/(stdziP*sqrt(2)))));

- exp(mpl)        + (1-alphaP)*exp(mc)*exp(yn)/exp(l);

- exp(mplzc)      + exp(mpl)*(exp(zc)/exp(zav))^exponenpfP;

- exp(mpk)        + alphaP*exp(mc)*exp(yn)/exp(k(-1));

- exp(surplusF)   + exp(phi) + fc;
  
- exp(rk)         + exp(mpk);

- exp(cm)         + exp(c) - hpP*(1-exp(l));

- exp(uc)         + exp(c)^-gammaP;


//% Exogenous Processes
- z               + rhozP*z(-1) - epsZ; 

- ub              + ub_SS;

- fc              + fc_SS;

- fr              + fr_SS;


//% Additional variables (not needed to solve the model)
- exp(seprate)         + lambdaP + exp(rho)*(1-lambdaP)+ deltafP*(1-lambdaP)*(1-exp(rho));//lambdaP + exp(rho)*(1-lambdaP)+ deltafP*(1-lambdaP)*(1-exp(rho));

//- exp(gdpIncome)  + exp(w)*exp(l) + exp(rk)*exp(k(-1)) + exp(N)*exp(d);

- exp(d_nosub) + exp(rpD)*exp(yfirm) - exp(w)*exp(lfirm) - vcP*exp(vfirm) - exp(mpk)*exp(kfirm) - exp(rho)*exp(lfirm)*fc/(1-exp(rho));

- exp(gdpIncome)  + exp(w)*exp(l) + exp(rk)*exp(k(-1)) + exp(N)*exp(d_nosub);

- exp(ie)         + exp(Ne)*(fetechP + fr);

- exp(gdpIncomedata) + exp(gdpIncome)/exp(rpD);

- exp(cmdata)     + exp(cm)/exp(rpD);

- exp(ikdata)     + exp(ik)/exp(rpD);

- exp(wdata)      + exp(w)/exp(rpD);

- exp(cdata)      + exp(c)/exp(rpD);

- exp(iedata)     + exp(ie)/exp(rpD);

end;


initval;
ub                  = ub_SS;
fc                  = fc_SS;
fr                  = fr_SS;
z                   = log(z_SS);
c                   = log(c_SS); 
l                   = log(l_SS);
u                   = log(u_SS);
q                   = log(q_SS);
s                   = log(s_SS);
v                   = log(v_SS);
m                   = log(m_SS);
mc                  = log(mc_SS);
rho                 = log(rho_SS);
zavInt              = log(zavInt_SS);
zav                 = log(zav_SS);
zc                  = log(zc_SS);
surplusF            = log(surplusF_SS);
surplusW            = log(surplusW_SS);
oo                  = log(oo_SS);
phi                 = log(phi_SS);
ik                  = log(ik_SS);
rk                  = log(rk_SS);
mpk                 = log(mpk_SS);
mpl                 = log(mpl_SS);
y                   = log(y_SS);
k                   = log(k_SS);
adjI                = adjI_SS;
multR               = log(multR_SS);
e                   = log(e_SS);
d                   = log(d_SS);
N                   = log(N_SS);
Ne                  = log(Ne_SS);
rpD                 = log(rpD_SS);
elasticity          = log(elasticity_SS);
markup              = log(markup_SS);
yfirm               = log(yfirm_SS);
yn                  = log(yn_SS);
lfirm               = log(lfirm_SS);
kfirm               = log(kfirm_SS);
vfirm               = log(vfirm_SS);
//Util                = log(c_SS) ;
//Welf                = (1/(1-betaP))*Util;
uc                  = log((exp(c))^-gammaP);
w                   = log(w_SS);
mplzc               = log(exp(mpl)*(exp(zc)/exp(zav))^exponenpfP);
wzc                 = log(exp(w) - etaP*(exp(mpl) - exp(mplzc)));
seprate             = log(lambdaP + (rho_SS)*(1-lambdaP)+ deltafP*(1-lambdaP)*(1-rho_SS));
cm                  = log(cm_SS);
gdpIncome           = log(gdpIncome_SS);
cmdata              = log(exp(cm)/exp(rpD));
gdpIncomedata       = log(exp(gdpIncome)/exp(rpD));
ikdata              = log(exp(ik)/exp(rpD));
wdata               = log(exp(w)/exp(rpD));
cdata               = log(exp(c)/exp(rpD));
ie                  = log(exp(Ne)*(fetechP + fr));
iedata              = log(exp(ie)/exp(rpD));
tau                 = log(tau_SS);
profitsub           = log(profitsub_SS);
d_nosub             = log(d_nosub_SS);
end;


resid(1);

if max(abs(resid))>1e-10
    display('RESIDUALS IN THE CYCLE_IRF.MOD FILE ARE NOT ZERO')
    keyboard
end


shocks;
var epsZ; stderr stdzP;
end;


planner_objective(log(c));
ramsey_policy(planner_discount=betaP,order=1);

stoch_simul(order=1,irf=20,nograph,noprint) cmdata u ikdata gdpIncomedata wdata N iedata markup; 

